/**
 * 
 */
package explorer;

import java.util.Random;
import java.util.Vector;


/**
 * This class implementsa configurable entry point.
 * The grid is created with walls on the perimeter.
 * Forbidden cells, simple and advanced explorer are placed in a random way and the number is configurable
 * @author andreamonaldini
 *
 */
public class ConfigurableSimulation {
	int dimension;
	int forbiddenCells;
	int simpleN;
	int advancedN;
	String theory;
	String specFile;
	String tcName;
	Random generator;
	Vector<CoordinateXY> simpleAgentsCoordinateVector;
	Vector<CoordinateXY> advancedAgentsCoordinateVector;
	/**
	 * Starts a new Simulation
	 * @param args 1st: grid dimension; 2nd: number of forbidden cells; 3rd: number of simple explorers; 4th: number of advanced explorers; 5th: prolog theory file; 6th: ReSpect theory file; 7th: Tuple Centre Id 
	 * 
	 */
	public static void main(String[] args) {
		if (args.length!=7){
			System.out.println("args: gridDimension numberOfForbiddenCells numberOfSimpleExplorerAgents numberOfAdvancedExplorerAgents prologTheoryFile respectFile tupleCentreId");
			System.exit(0);
		}else{
			new ConfigurableSimulation(	(new Integer(args[0])).intValue(),(new Integer(args[1])).intValue(),
					(new Integer(args[2])).intValue(),(new Integer(args[3])).intValue(),args[4],args[5],args[6]);
		}
	}
	
	public ConfigurableSimulation (int dimension, int forbiddenCells, int simpleN, int advancedN, String theory, String specFile, String tcName ){
		this.dimension=dimension;
		this.forbiddenCells=forbiddenCells;
		this.simpleN=simpleN;
		this.advancedN=advancedN;
		this.theory=theory;
		this.specFile=specFile;
		this.tcName=tcName;
		this.generator=new Random();
		CoordinateXY coord;
		int xCoord;
		int yCoord;
		int count;
		CoordinateXY[] simpleExplorerCoordinates=null;
		int[] simpleExplorerDirections=null;
		CoordinateXY[] advancedExplorerCoordinates=null;
		//grid creation
		int cells[][]=new int[dimension][dimension];
		System.out.println("Grid Dimension: "+dimension);
		for(int x=0;x<dimension;x++){
			for(int y=0;y<dimension;y++){
				if ((x==0)||(y==0)||(x==(dimension-1))||(y==(dimension-1))){
					cells[x][y]=SpaceCell.WALL;
					System.out.println("Wall at ("+x+","+y+").");
				}else{
					cells[x][y]=SpaceCell.ACCESSIBLE;
				}
			}
		}
		count=0;
		
		//forbidden cells creation
		while(count<this.forbiddenCells){
			coord=randomCoordinate(this.dimension);
			xCoord=coord.getX();
			yCoord=coord.getY();			
			if ((cells[xCoord][yCoord]!=SpaceCell.WALL)&&(cells[xCoord][yCoord]!=SpaceCell.FORBIDDEN)){
				cells[xCoord][yCoord]=SpaceCell.FORBIDDEN;
				System.out.println("Forbidden cell at ("+xCoord+","+yCoord+").");
				count++;
			}
		}
		
		//simple agents creation
		simpleAgentsCoordinateVector=new Vector<CoordinateXY>();
		count=0;
		simpleExplorerCoordinates=new CoordinateXY[simpleN];
		while(count<this.simpleN){
			coord=randomCoordinate(this.dimension);
			xCoord=coord.getX();
			yCoord=coord.getY();
			if ((cells[xCoord][yCoord]!=SpaceCell.WALL)&&(cells[xCoord][yCoord]!=SpaceCell.FORBIDDEN)&&(!usedCoordinate(this.simpleAgentsCoordinateVector,coord))){
				simpleAgentsCoordinateVector.add(coord);
				simpleExplorerCoordinates[count]=coord;
				System.out.println("Simple Explorer Agent at ("+xCoord+","+yCoord+").");
				count++;
			}
		}
		
		//direction creation
		count=0;
		simpleExplorerDirections=new int[simpleN];
		while(count<this.simpleN){
			simpleExplorerDirections[count]=generator.nextInt(4);
			count++;
		}
		//end of simple agents creation
		
		
		//advanced agents creation
		advancedAgentsCoordinateVector=new Vector<CoordinateXY>();
		count=0;
		advancedExplorerCoordinates=new CoordinateXY[advancedN];
		while(count<this.advancedN){
			coord=randomCoordinate(this.dimension);
			xCoord=coord.getX();
			yCoord=coord.getY();
			if ((cells[xCoord][yCoord]!=SpaceCell.WALL)&&(cells[xCoord][yCoord]!=SpaceCell.FORBIDDEN)&&(!usedCoordinate(this.simpleAgentsCoordinateVector,coord))&&(!usedCoordinate(this.advancedAgentsCoordinateVector,coord))){
				advancedAgentsCoordinateVector.add(coord);
				advancedExplorerCoordinates[count]=coord;
				System.out.println("Advanced Explorer Agent at ("+xCoord+","+yCoord+").");
				count++;
			}
		}
		
		new Simulation(dimension,cells,simpleExplorerCoordinates,simpleExplorerDirections, advancedExplorerCoordinates, theory, specFile, tcName);
	}
	
	private CoordinateXY randomCoordinate(int dimension){
		int x=this.generator.nextInt(dimension);
		int y=this.generator.nextInt(dimension);
		return new CoordinateXY(x,y);
	}
	
	private boolean usedCoordinate(Vector<CoordinateXY> vector, CoordinateXY coord){
		int x=coord.getX();
		int y=coord.getY();
		boolean found=false;
		for (int i=0;i<vector.size();i++){
			if ((vector.elementAt(i).getX()==x)&&(vector.elementAt(i).getY()==y)){
				found=true;
				break;
			}
		}
		return found;
		
	}
}
