/**
 * 
 */
package explorer;

import alice.logictuple.LogicTuple;
import alice.logictuple.Value;
import alice.logictuple.Var;
import alice.tucson.api.AgentId;
import alice.tucson.api.TucsonException;
import alice.tucson.api.TupleCentreId;

/**
 * This class implements a Simple Explorer Agent
 * @author andreamonaldini
 *
 */
public class SimpleExplorerAgent extends ExplorerAgent {
	TupleCentreId tc; //tuple centre for agents coordination
	int direction;	//the movement direction
	
	/**
	 * Constructs a new Simple Explorer Agent 
	 * @param id Agent ID.
	 * @param grid The virual space where agent works.
	 * @param x The x position of the agent
	 * @param y The y position of the agent
	 * @param direction The movement direction (ExplorerAgent.NORT-EAST-SOUTH-WEST)
	 * @throws TucsonException
	 */
	public SimpleExplorerAgent(AgentId id, SpaceGrid grid, int x, int y, int direction)
	throws TucsonException {
		super(id, grid, x, y);
		this.direction=direction;
		this.tc=null;
		
		
		
	}
	/**
	 * Constructs a new Simple Explorer Agent 
	 * @param id Agent ID.
	 * @param grid The virual space where agent works.
	 * @param x The x position of the agent
	 * @param y The y position of the agent
	 * @param direction The movement direction (ExplorerAgent.NORT-EAST-SOUTH-WEST)
	 * @param tc The tuple centre id
	 * @throws TucsonException
	 */
	public SimpleExplorerAgent(AgentId id, SpaceGrid grid, int x, int y, int direction, TupleCentreId tc)
	throws TucsonException {
		super(id, grid, x, y);
		this.direction=direction;
		this.tc=tc;
		
		
		
	}
	
	/* (non-Javadoc)
	 * @see alice.tucson.api.Agent#body()
	 */
	@Override
	protected void body() {
		boolean moved;	//true if the agent has moved
		int antiLoop=0;
		try{
			if (tc==null){
				tc=new TupleCentreId("tc");
			}
			
			//Positioning Syncronization
			rd(this.tc, new LogicTuple("startPositioning"));
			
			//output of the agent position
			//template explorerAt(X, Y, AgentID)
			
			out(this.tc,new LogicTuple("explorerAt", new Value(this.x),new Value(this.y), new Value(this.id.toString())));
			System.out.println("SimpleExplorerAgent "+this.id+" placed at "+this.x+","+this.y);
			
			//the cell where the agent is initially places is already explored...
			//template cell(X,Y)
			//Waiting syncronization
			rd(this.tc, new LogicTuple("start"));
			out(tc, new LogicTuple("cell", new Value(this.x),new Value(this.y)));
			
			//agent control flow - the agent's work terminate when all cells are explored
			while(rdp(this.tc, new LogicTuple("virtualSpaceExplored"))==null){
				
				//Wait for the right round
				in(this.tc,new LogicTuple("resource_token",new Value(id.toString())));
				
				moved=false;
				
				//the agent turn around in every direction
				for (int i=0;i<4;i++){
					
					//if in the next cell: there is a wall, there is another agent, is forbidden, is already explored
					//turn clockwise and make the same checks
					if ((this.nextCellType()==SpaceCell.WALL)||
							(rdp(tc, new LogicTuple("explorerAt", new Value(this.nextCellX()),new Value(this.nextCellY()), new Var("_")))!=null)||
							(this.nextCellType()==SpaceCell.FORBIDDEN)||
							(rdp(tc, new LogicTuple("cell", new Value(this.nextCellX()),new Value(this.nextCellY())))!=null)){
						
						//if the nex cell is inaccessible put the tuple forbiddenCell(nexCellX,nextCellY) if the forbidden cell wasn't already discovered
						if (this.nextCellType()==SpaceCell.FORBIDDEN){
							
							//if the forbidden cell wasn't already discovered or already assigned to an Advanced Explorer...
							if ((rdp(this.tc, new LogicTuple("forbiddenCell",new Value(this.nextCellX()),new Value(this.nextCellY())))==null)&&
									(rdp(this.tc, new LogicTuple("forbiddenCellAssigned",new Value(this.nextCellX()),new Value(this.nextCellY())))==null)&&
									(rdp(this.tc, new LogicTuple("cell",new Value(this.nextCellX()),new Value(this.nextCellY())))==null)){
								//declare it forbiddenCell
								out(tc,new LogicTuple("forbiddenCell",new Value(this.nextCellX()),new Value(this.nextCellY())));
								System.out.println("Forbidden Cell found at "+this.nextCellX()+","+this.nextCellY());
							}
						}
						this.turnClockwise();
						
						
					}else{
						this.goForward();
						//here only if the new cell is never accessed before
						//output of the tuple cell(X,Y)
						out(tc, new LogicTuple("cell", new Value(this.x),new Value(this.y)));
						moved=true;
						antiLoop=0;
						break;
					}
				}
				if (!moved){
					//here if there aren't inexplored cells near the agent - 
					if (antiLoop==4){	//anti Loop behavior
						this.turnClockwise();
						antiLoop=0;
					}else{	//normal behavior
						this.turnClockwise();
						this.turnClockwise();
						this.turnClockwise();
					}
					
					//all adiacent cells are re-analyzed and the first accessible (already explored) is selected
					for (int i=0;i<4;i++){
						if ((this.nextCellType()==SpaceCell.WALL)||
								(rdp(tc, new LogicTuple("explorerAt", new Value(this.nextCellX()),new Value(this.nextCellY()), new Var("_")))!=null)||
								((this.nextCellType()==SpaceCell.FORBIDDEN)&&(rdp(tc, new LogicTuple("cell", new Value(this.nextCellX()),new Value(this.nextCellY())))==null))){
							this.turnClockwise();
						}else{
							this.goForward();
							antiLoop++;
							break;
						}
					}
				}
				
				out(this.tc,new LogicTuple("resource_token",new Value(id.toString())));
				System.out.flush();
			}
			//remove the last position tuple 
			in(this.tc, new LogicTuple("explorerAt", new Value(this.x),new Value(this.y), new Value(this.id.toString())));	
			System.out.println("SimpleExplorerAgent "+this.id+" exited - total explored cells = "+
					rdp(this.tc, new LogicTuple("exploredCell",new Var("X"))).getVarValue("X").intValue());
		}catch(Exception e){System.out.println("SimpleExplorerAgent - Control error");}
		
	}
	
	private int nextCellType(){
		switch (this.direction) {
		case ExplorerAgent.NORTH: return this.grid.cellType(this.x,this.y+1);
		case ExplorerAgent.EAST: return this.grid.cellType(this.x+1,this.y);
		case ExplorerAgent.SOUTH: return this.grid.cellType(this.x,this.y-1);
		case ExplorerAgent.WEST: return this.grid.cellType(this.x-1,this.y);
		default: return -1;
		}
	}
	private int nextCellX(){
		switch (this.direction) {
		case ExplorerAgent.NORTH: return this.x;
		case ExplorerAgent.EAST: return this.x+1;
		case ExplorerAgent.SOUTH: return this.x;
		case ExplorerAgent.WEST: return this.x-1;
		default: return -1;
		}
	}
	
	private int nextCellY(){
		switch (this.direction) {
		case ExplorerAgent.NORTH: return this.y+1;
		case ExplorerAgent.EAST: return this.y;
		case ExplorerAgent.SOUTH: return this.y-1;
		case ExplorerAgent.WEST: return this.y;
		default: return -1;
		}
	}
	
	
	/**
	 * Turn the movement direction clockwise
	 */
	private void turnClockwise(){
		switch (this.direction) {
		case ExplorerAgent.NORTH: this.direction=ExplorerAgent.EAST;break;
		case ExplorerAgent.EAST: this.direction=ExplorerAgent.SOUTH;break;
		case ExplorerAgent.SOUTH: this.direction=ExplorerAgent.WEST;break;
		case ExplorerAgent.WEST: this.direction=ExplorerAgent.NORTH;break;
		}
	}
	
	private void goForward(){
		try{
			//remove the old position tuple explorerAt(X,Y,AgentId)
			in(this.tc, new LogicTuple("explorerAt", new Value(this.x),new Value(this.y), new Value(this.id.toString())));		
		}catch(Exception e){System.out.println("SimpleExplorerAgent - Error removing old position tuple.");}
		
		switch (this.direction) {
		case ExplorerAgent.NORTH:  this.y=this.y+1;break;
		case ExplorerAgent.EAST: this.x=this.x+1;break;
		case ExplorerAgent.SOUTH: this.y=this.y-1;break;
		case ExplorerAgent.WEST: this.x=this.x-1;break;
		}
		//add the new position tuple explorerAt(X,Y,AgentId)
		try {
			out(this.tc, new LogicTuple("explorerAt", new Value(this.x),new Value(this.y), new Value(this.id.toString())));
			System.out.println("SimpleExplorerAgent "+this.id+" moved to "+this.x+","+this.y);
		} catch (Exception e){System.out.println("SimpleExplorerAgent - Error adding new position tuple.");}	
	}
	
	
	
}
