/**
 * 
 */
package explorer;


/**
 * This class implements a simple simulation entry point. 
 * The simulation grid is 8x8 with walls in the perimeter and 3 forbidden cells at (1,3), (4,2), (3,4). 
 * There are three agents on the grid: two simple explorers and two advanced explorers. 
 * SimpleExplorerAgents#1: at (3,1) - direction WEST. 
 * SimpleExplorerAgent#2: at (5,1) - direction EAST. 
 * AdvancedExplorerAgent#1: at (4,1). 
 * AdvancedExplorerAgent#2: at (1,6). 
 * The default TupleCentreId used in the simulation is "explorerTC" but the name can be specified as parameter. 
 * NB: the two text files must be in the same directory of the jar file if the jar version is used. 
 * @author andreamonaldini
 *
 */
public class SimpleSimulation {
	
	/**
	 * Starts a new Simulation
	 * @param args 1st arg: prolog theory file; 2nd arg: ReSpect theory file; 3rd arg: Tuple Centre Id
	 */
	public static void main(String[] args) {
		String theory=null;
		String specFile=null;
		String tcName=null;;
		if (args.length==0){
			System.out.println("args: prologTheoryFile respectSpecificationFile TupleCentreId");
			System.exit(0);
		}else{
			
			theory=args[0];
			specFile=args[1];
			tcName=args[2];
			System.out.println("TupleCentreId used: "+tcName);
		}
		//grid creation
		int dimension=8;
		int cells[][]=new int[dimension][dimension];
		for(int x=0;x<dimension;x++){
			for(int y=0;y<dimension;y++){
				if ((x==0)||(y==0)||(x==(dimension-1))||(y==(dimension-1))){
					cells[x][y]=SpaceCell.WALL;
				}else{
					cells[x][y]=SpaceCell.ACCESSIBLE;
				}
			}
		}
		
		//Three forbidden cells
		cells[1][3]=SpaceCell.FORBIDDEN;
		cells[4][2]=SpaceCell.FORBIDDEN;
		cells[3][4]=SpaceCell.FORBIDDEN;
		
		//agents creation
		//simple agents
		CoordinateXY[] simpleExplorerCoordinates=new CoordinateXY[2];
		simpleExplorerCoordinates[0]=new CoordinateXY(3,1);
		simpleExplorerCoordinates[1]=new CoordinateXY(5,1);
		
		int[] simpleExplorerDirections= new int[2];
		simpleExplorerDirections[0]=ExplorerAgent.WEST;
		simpleExplorerDirections[1]=ExplorerAgent.EAST;
		
		//advanced agents
		CoordinateXY[] advancedExplorerCoordinates=new CoordinateXY[2];
		advancedExplorerCoordinates[0]=new CoordinateXY(4,1);
		advancedExplorerCoordinates[1]=new CoordinateXY(1,6);
		
		
		
		new Simulation(dimension,cells,simpleExplorerCoordinates,simpleExplorerDirections, advancedExplorerCoordinates, theory, specFile, tcName);
		
		
	}
	
}

