/**
 * 
 */
package explorer;

import java.util.Vector;

import alice.tucson.api.Agent;
import alice.tucson.api.AgentId;
import alice.tucson.api.InvalidAgentIdException;
import alice.tucson.api.InvalidTupleCentreIdException;
import alice.tucson.api.TucsonException;
import alice.tucson.api.TupleCentreId;

/**
 * Entry point for all the simulations
 * @author andreamonaldini
 *
 */
public class Simulation {
	SpaceGrid spaceGrid;
	Vector<SimpleExplorerAgent> simpleExplorerAgents;
	Vector<AdvancedExplorerAgent> advancedExplorerAgents;
	
	TupleCentreId tc;
	/**
	 * Creates a new simulation environment
	 * @param dimension The dimension (row or column) the grid.
	 * @param cells Cell Matrix: value in (ACCESSIBLE, FORBIDDEN, WALL)
	 * @param simpleExplorerCoordinates Array of CoordinateXY containing the Simple Explorer Agent's coordinates
	 * @param simpleExplorerDirections Array of directions (ExplorerAgent.NORT-EAST-SOUTH-WEST) for Simple Explorer Agents
	 * @param advancedExplorerCoordinates Array of CoordinateXY containing the Advanced Explorer Agent's coordinates
	 * @param theory Prolog theory for path planning
	 * @param specFile ReSpect specification tuples file.
	 * @param tcName Tuple centre Id.
	 */
	public Simulation(int dimension, int cells[][], CoordinateXY[] simpleExplorerCoordinates,
						int[] simpleExplorerDirections, 
						CoordinateXY[] advancedExplorerCoordinates,
						String theory,
						String specFile,
						String tcName) {
		super();
		try {
			tc = new TupleCentreId(tcName);
		} catch (InvalidTupleCentreIdException e1) {
			e1.printStackTrace();
		}
		this.spaceGrid=new SpaceGrid(dimension,cells);
		
		//Save the reference to created agents to two vectors 
		this.simpleExplorerAgents=new Vector<SimpleExplorerAgent>();
		this.advancedExplorerAgents=new Vector<AdvancedExplorerAgent>();
		//SimpleExplorerAgents creation
		for(int i=0;i<simpleExplorerCoordinates.length;i++){
			try {
				SimpleExplorerAgent agent=new SimpleExplorerAgent(new AgentId("simpleExplorerAgent_"+i),this.spaceGrid,simpleExplorerCoordinates[i].getX(),
						simpleExplorerCoordinates[i].getY(),simpleExplorerDirections[i],tc);
				this.simpleExplorerAgents.add(agent);
				agent.spawn();
			} catch (InvalidAgentIdException e) {
				e.printStackTrace();
			} catch (TucsonException e) {
				e.printStackTrace();
			}
		}
		//AdvancedExplorerAgents creation
		for(int i=0;i<advancedExplorerCoordinates.length;i++){
			try {
				AdvancedExplorerAgent agent=new AdvancedExplorerAgent(new AgentId("advancedExplorerAgent_"+i),this.spaceGrid,advancedExplorerCoordinates[i].getX(),
						advancedExplorerCoordinates[i].getY(),theory,tc);
				this.advancedExplorerAgents.add(agent);
				agent.spawn();
			} catch (InvalidAgentIdException e) {
				e.printStackTrace();
			} catch (TucsonException e) {
				e.printStackTrace();
			}
		}
		try {
			//The agent spec is useful to set up the environment
			Agent ag= new SpecAgent(new AgentId("spec"), this.spaceGrid, specFile,tc,advancedExplorerCoordinates.length);
			ag.spawn();
		} catch (InvalidAgentIdException e) {
			e.printStackTrace();
		} catch (TucsonException e) {
			e.printStackTrace();
		}
	}	
}
