/**
 * 
 */
package explorer;

/**
 * This class implements a cell to be inserted in a SpaceGrid grid
 * @author andreamonaldini
 *
 */
public class SpaceCell {
	
	/**
	 * An accessible cell
	 */
	public final static int ACCESSIBLE=0;
	
	/**
	 * A forbidden cell (not accessible to all explorators) 
	 */
	public final static int FORBIDDEN=1;
	
	/**
	 *A wall 
	 */
	public final static int WALL=2;
	
	/**
	 * The cell type (ACCESSIBLE, FORBIDDEN, WALL)
	 */
	int cellType;	//contains the cell type (ACCESSIBLE, FORBIDDEN, WALL)

	/**
	 * Create a new cell 
	 * @param cellType cell type (ACCESSIBLE, FORBIDDEN, WALL)
	 */
	public SpaceCell(int cellType) {
		super();
		this.cellType = cellType;
	}

	/**
	 * Check if the cell is accessible.
	 * @return If the cell is accessible.
	 */
	public boolean isAccessible() {
		return cellType==SpaceCell.ACCESSIBLE;
	}
	
	/**
	 * Check if the cell is forbidden
	 * @return If the cell is forbidden.
	 */
	public boolean isForbidden() {
		return cellType==SpaceCell.FORBIDDEN;
	}
	
	/**
	 * Check if the cell is a wall
	 * @return If the cell is a wall.
	 */
	public boolean isWall() {
		return cellType==SpaceCell.WALL;
	}
	

	/**
	 * Sets the cell type (SpaceCell.ACCESSIBLE - SpaceCell.FORBIDDEN - SpaceCell.WALL)
	 * @param cellType The cell type to set (SpaceCell.ACCESSIBLE - SpaceCell.FORBIDDEN - SpaceCell.WALL)
	 */
	public void setCellType(int cellType) {
		this.cellType = cellType;
	}
	
	/**
	 * Check the cell type
	 * @return The cell type (SpaceCell.ACCESSIBLE - SpaceCell.FORBIDDEN - SpaceCell.WALL)
	 */
	public int cellType(){
		return this.cellType;
	}
	
	
	

}
