/**
 * 
 */
package explorer;


/**
 * Implements a 2D virtual space (grid)
 * @author andreamonaldini
 *
 */
public class SpaceGrid {
	int dimension;	//grid dimension
	SpaceCell cells[][];	//cellMatrix
	
	/**
	 * Creates a 2D virtual space - a square grid of X-Y cells (for both the axes the numeration start in the bottom-left corner)
	 * @param dimension The heigh (x or y-size) of the grid.
	 * @param cells Cell Matrix: value in (ACCESSIBLE, FORBIDDEN, WALL)
	 */
	public SpaceGrid(int dimension, int cells[][]) {
		super();
		this.dimension = dimension;
		this.cells=new SpaceCell[dimension][dimension];
		for (int x=0; x<this.dimension; x++){
			for (int y=0; y<this.dimension; y++){
				this.cells[x][y]=new SpaceCell(cells[x][y]);
			}
		}
	}
	
	/**
	 * Returns the cell type (ACCESSIBLE, FORBIDDEN, WALL)
	 * @param x Row Coordinate in the grid.
	 * @param y Column Coordinate in the grid.
	 * @return The cell type.
	 */
	int cellType(int x, int y){
		return cells[x][y].cellType();
	}


	/**
	 * Returns the dimension of the grid 
	 * @return Returns the dimension of the grid
	 */
	public int getDimension() {
		return this.dimension;
	}
	
	/**
	 * Returns the number of wall-cell in the grid
	 * @return Returns the number of wall-cell in the grid
	 */
	public int wallNumber(){
		int count=0;
		for (int i=0;i<this.dimension;i++){
			for (int j=0;j<this.dimension;j++){
				if (this.cells[i][j].cellType()==SpaceCell.WALL){
					count++;
				}
			}
		}
		return count;
	}

}
