/**
 * 
 */
package explorer;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import alice.logictuple.LogicTuple;
import alice.logictuple.Value;
import alice.tucson.api.Agent;
import alice.tucson.api.AgentId;
import alice.tucson.api.TucsonException;
import alice.tucson.api.TupleCentreId;

/**
 * The SpecAgent is a simple agent used to set up specification tuples and initial syncronization tuples on tuple centre.
 * @author andreamonaldini
 *
 */
public class SpecAgent extends Agent {
	FileInputStream specFile;		//ReSpecT spcification file
	String spec; //ReSpecT spcification string
	SpaceGrid grid;
	TupleCentreId tc;
	int numberOfAdvancedExplorerAgents;

	/**
	 * Creates a new agent for tuple centre initialization
	 * @param arg0 The agent ID
	 * @param grid The virual space where agent works.
	 * @param specFile ReSpect specification file for tuple centre reactions
	 * @param numberOfAdvancedExplorerAgents Number of AdvancedExplorerAgent in the system
	 * @throws TucsonException
	 */
	public SpecAgent(AgentId arg0,SpaceGrid grid,FileInputStream specFile, int numberOfAdvancedExplorerAgents) throws TucsonException {
		super(arg0);
		this.specFile=specFile;
		this.grid=grid;
		this.tc=null;
		this.numberOfAdvancedExplorerAgents=numberOfAdvancedExplorerAgents;
	}
	/**
	 * Creates a new agent for tuple centre initialization
	 * @param arg0 The agent ID
	 * @param grid The virual space where agent works.
	 * @param specFile ReSpect specification file for tuple centre reactions
	 * @param tc The tuple centre id
	 * @param numberOfAdvancedExplorerAgents Number of AdvancedExplorerAgent in the system
	 * @throws TucsonException
	 */
	public SpecAgent(AgentId arg0,SpaceGrid grid,String specFile, TupleCentreId tc, int numberOfAdvancedExplorerAgents) throws TucsonException {
		super(arg0);
		try {
			this.specFile=new FileInputStream(specFile);
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		this.grid=grid;
		this.tc=tc;
		this.numberOfAdvancedExplorerAgents=numberOfAdvancedExplorerAgents;
	}

	/* (non-Javadoc)
	 * @see alice.tucson.api.Agent#body()
	 */
	@Override
	protected void body() {
		//read the ReSpecT specifications

		BufferedReader br=new BufferedReader(new InputStreamReader(this.specFile));
		spec="";
		String s;
		try {
			while((s=br.readLine())!=null){
				spec=spec.concat(s);
				spec=spec.concat("\n");
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("##### ReSpect ####");
		System.out.println(spec);
		System.out.println("##### End ReSpect ####");
		
		//ReSpect
		try {
			if (tc==null){
				tc=new TupleCentreId("tc");
			}
				setSpec(tc,spec);
				
				//Tuple for barrier syncronization - end of exploration
				out(tc,new LogicTuple("exploredCell",new Value(0)));
				out(tc,new LogicTuple("gridDimension",new Value(this.grid.getDimension()*this.grid.getDimension()-this.grid.wallNumber())));
				
				//Tuples for turn
				out(tc,new LogicTuple("tickets",new Value(0)));
				out(tc,new LogicTuple("current_turn",new Value(0)));
				//Tuple for prolog syncronization
				out(tc,new LogicTuple("advancedAgentNumber",new Value(0)));
				
				System.out.println("Set up Specification Tuples (ReSpect)...");
				System.out.println("There are "+(this.grid.getDimension()*this.grid.getDimension()-this.grid.wallNumber())+" cells to be explored.");
				
				//Tuple for start the positioning of explorers on the grid
				out(tc,new LogicTuple("startPositioning"));
				
				//Syncronization step - proceed only if all advancedAgents have join the system 
				rd(tc,new LogicTuple("advancedAgentNumber",new Value(numberOfAdvancedExplorerAgents)));
				
				//Start Exploration
				out(tc,new LogicTuple("start"));
				System.out.println("Exploration is started - all explorers spawn");
				//rd(tc, new LogicTuple("virtualSpaceExplored"));
		} catch (Exception e) {
			e.printStackTrace();
		}
		

	}

}
